/**
 * ESMDataExtractor reads in an ESM file and outputs an xml file
 * that maps unique ids to names to unique_id.jpg.
 */

#include <iostream>
#include <fstream>
#include <cassert>
#include "EsmFile.h"
#include "utils.h"

using namespace std;

void printRecords(TCHAR* type, std::ofstream* file, CEsmFile* pFile) {
	int arrayIndex = 0;
	CEsmRecord* pRecord = pFile->FindFirst(type, arrayIndex);
	while(pRecord != NULL) {
		*file << "<object id=\"";
		*file << pRecord->GetFieldString(ESM_FIELD_ID) << "\" name=\"";
		*file << pRecord->GetFieldString(ESM_FIELD_NAME) << "\" image=\"";
		*file << pRecord->GetFieldString(ESM_FIELD_ID) << ".jpg\"/>" << endl;
		pRecord = pFile->FindNext(type, arrayIndex);
	}
}

void ProcessFile(CEsmFile* pFile) {
	std::ofstream file;
	file.open("output.xml", std::ios::out);
	CEsmRecord* pRecord;
	int arrayIndex = 0;
	  
	file << "<?xml version=\"1.0\" ?>" << endl;
	file << "<ObjectData>" << endl;
	//write out weapons
	printRecords(MWESM_REC_WEAP, &file, pFile);

	//write out containers
	printRecords(MWESM_REC_CONT, &file, pFile);
	
	//write out misc
	printRecords(MWESM_REC_MISC, &file, pFile);
	
	//write out armor
	printRecords(MWESM_REC_ARMO, &file, pFile);

	//write out clothing
	printRecords(MWESM_REC_CLOT, &file, pFile);

	//write out ingrediants
	printRecords(MWESM_REC_INGR, &file, pFile);

	//write out books
	printRecords(MWESM_REC_BOOK, &file, pFile);

	file << "</ObjectData>";
}

int main(void) {
	CEsmFile pFile;
	pFile.Read("Morrowind.esm");
	ProcessFile(&pFile);
	return 0;
}