#include "game_container.h"

GameContainer::GameContainer(CEsmContainer* pEsmContainer, const TCHAR* pName) : GameObject(pName) {
	assert(pEsmContainer != NULL);
	this->pEsmContainer = pEsmContainer;
}

/**
 * Adds a GameObject using name equality and updates count
 */
void GameContainer::add(GameObject* pObject, int count) {
	assert(pObject != NULL && count > 0);
	
	map<const TCHAR*, int>::iterator it = container.find(pObject->getName());
	if (it !=  container.end()) {  // already exists, update count
		it->second += count;
	} else {					   // otherwise, add it
		container.insert(make_pair(pObject->getName(), count));
	}
}

/**
 * Removes GameObject by name equality and updates count  (i.e. decrement the count)
 * If trying to remove more objects than it the container contains, remove them all.
 */
void GameContainer::remove(GameObject* pObject, int count) {
	assert(count > 0);
	
	if (this->count(pObject) > 0) {
		map<const TCHAR*, int>::iterator it = container.find(pObject->getName());
		assert(it != container.end());   // if this trips, it would be a bug
		(*it).second -= count;
		if ((*it).second <= 0) {
			container.erase(pObject->getName());	
		}
	}
}

void GameContainer::clear() {
	container.clear();
}

bool GameContainer::contains(GameObject* pObject) {
	return (container.count(pObject->getName()) > 0);
}

int GameContainer::size() {
	return container.size();
}

int GameContainer::count(GameObject* pObject) {
	map<const TCHAR*, int>::iterator it = container.find(pObject->getName());
	if (it != container.end()) {
		return (*it).second;
	} else {
		return 0;
	}
}

bool GameContainer::equals(GameObject* pObject) {
	if (typeid(this) == typeid(pObject) && GameObject::equals(pObject)) {
		GameContainer* pContainer = dynamic_cast<GameContainer*>(pObject);
		return (this->pEsmContainer == pContainer->pEsmContainer &&
				this->container == pContainer->container);
	}
	return false;
}

// output methods
CEsmContainer* GameContainer::toESMContainer() {
	CEsmContainer* pCopy = (CEsmContainer*)CEsmContainer::Create();
	pEsmContainer->Copy(pCopy);
	map<const TCHAR*, int>::iterator it;
	for( it = container.begin(); it != container.end(); it++ ) {
		const TCHAR* pName = (*it).first;
		int count = (*it).second;
		CEsmSubNPCO* pNPCO = (CEsmSubNPCO*)CEsmSubNPCO::Create();
		pNPCO->CreateNew();
		pNPCO->SetType(MWESM_SUBREC_NPCO);
		pNPCO->SetItem(pName);	
		pNPCO->SetCount(count);
		pCopy->AddSubRecord(pNPCO);
	}	
	return pCopy;
}