/**
 * Another ADT that inherits from GameObject.  It provides basic container functionality, such
 * as adding and deleting objects.  And like GameObject there is an output method toESMContainer()
 * that returns the appropriate ESM representation for this container.
 */

#ifndef _GAME_CONTAINER_H
#define _GAME_CONTAINER_H

#include <cassert>
#include <typeinfo>
#include <map>
#include "game_object.h"
#include "EsmContainer.h"

using namespace std;

typedef std::map<const TCHAR*, int> Container;		// a map of GameObject names to counts

class GameContainer : public GameObject {
	Container container;
	CEsmContainer* pEsmContainer;

public:
	GameContainer(CEsmContainer* pEsmContainer, const TCHAR* pName);

	// standard container type methods
	void add(GameObject* pObject, int count);
	void remove(GameObject* pObject, int count);
	void clear();
	int size();
	int count(GameObject* pObject);
	bool contains(GameObject* pObject);

	bool equals(GameObject* pObject);

	// output methods
	CEsmContainer* toESMContainer();
};

#endif