#include "game_object.h"

void init_vect3d(vector3d& vect3d) {
	vect3d.x = 0;
	vect3d.y = 0;
	vect3d.z = 0;
}

bool equal_vect3d(vector3d v1, vector3d v2) {
	return (v1.x == v2.x && v1.y == v2.y && v1.z == v2.z);
}

GameObject::GameObject(const TCHAR* pName) : name(pName) {
	init_vect3d(pos);
	init_vect3d(rot);
	init_vect3d(size);
}

GameObject::GameObject(const TCHAR* pName, vector3d pos, vector3d rot, vector3d size) : name(pName) {
	setPos(pos);
	setRot(rot);
	setSize(size);
}

// for these placement methods:  first assume no rotation, then apply proper rotation afterwards
// in essence this means, objects have absolute tops, bottoms, lefts, rights, fronts, regardless of
// orientation.  so putAbove on a table always refers to the table top, even if that table happens to
// be upside down.  in that case, the object actually gets put "under" the table.  perhaps this will
// change, but i think this type of positioning makes sense, particularly in reference to doors, tables,
// chairs, and similar objects.

void GameObject::putAbove(GameObject* obj) {
	synchObjects(obj);
	setZpos(getZpos() + getZsize() + obj->getZsize());
}

void GameObject::putBelow(GameObject* obj) {
	synchObjects(obj);
	setZpos(getZpos() - getZsize() - obj->getZsize());
}

void GameObject::putLeft(GameObject* obj) {
	synchObjects(obj);
	setXpos(getXpos() + getXsize() + obj->getXsize());
}

void GameObject::putRight(GameObject* obj) {
	synchObjects(obj);
	setXpos(getXpos() - getXsize() - obj->getXsize());
}

void GameObject::putBehind(GameObject* obj) {
	synchObjects(obj);
	setYpos(getYpos() - getYsize() - obj->getYsize());
}

void GameObject::putInFront(GameObject* obj) {
	synchObjects(obj);
	setYpos(getYpos() + getYsize() + obj->getYsize());
}

bool GameObject::equals(GameObject* obj) {
	return (this->name == obj->name &&
			equal_vect3d(this->pos, obj->pos) &&
			equal_vect3d(this->rot, obj->rot) &&
			equal_vect3d(this->size, obj->size));
}

CEsmSubCellRef* GameObject::toCREF() {
	CEsmSubCellRef* cref = (CEsmSubCellRef*) CEsmSubCellRef::Create();
	cref->CreateNew();
	cref->SetRefName(name);

	CEsmSubPos6* pos = (CEsmSubPos6*) cref->FindSubRecord (MWESM_SUBREC_DATA);
	pos->SetPosX(getXpos());
	pos->SetPosY(getYpos());
	pos->SetPosZ(getZpos());
	pos->SetRotX(getXrot());
	pos->SetRotY(getYrot());
	pos->SetRotZ(getZrot());

	return cref;
}
