#ifndef _GRAPH_H_
#define _GRAPH_H_

#include <deque>
#include <map>
#include <set>
#include <cstring>

using namespace std;

const int UNKNOWN = -1;
const int IS_A = 1;					
const int CENTER_OF = 0x2;			
const int ON_TOP_OF = 0x4;			
const int AROUND = 0x8;				
const int BESIDE = 0x10;
const int EDGE_OF = 0x20;
const int CORNER_OF = 0x40;
const int BELOW = 0x80;
const int INSIDE = 0x100;

struct ltstr {
  bool operator()(const char* s1, const char* s2) const{
    return strcmp(s1, s2) < 0;
  }
};

typedef std::map<const char*, int, ltstr> edge_map;
typedef std::map<const char*, edge_map, ltstr> adjacency_map;

class Graph {
	mutable adjacency_map forward_map;
	mutable adjacency_map backward_map;

	bool hasPath(const char* from, const char* to, adjacency_map& amap) const;

public:
	void addNode(const char* node);
	void removeNode(const char* node);
	void addEdge(const char* from, const char* to, int weight);
	void removeEdge(const char* from, const char* to);

	bool hasNode(const char* node) const;
	bool hasEdge(const char* from, const char* to) const;
	bool hasForwardPath(const char* from, const char* to) const;
	bool hasBackwardPath(const char* from, const char* to) const;
	
	int edgeWeight(const char* from, const char* to) const;
	edge_map getAdjacent(const char* node) const;
	edge_map getAdjacentInv(const char* node) const;
	
	int inDegree(const char* node) const;
	int outDegree(const char* node) const;
	void print() const;
};

#endif