#ifndef _ROOM_H_
#define _ROOM_H_

#include <map>
#include <list>
#include <set>
#include <cstring>
#include <cmath>
#include "graph.h"
#include "game_object.h"
#include "EsmFile.h"

#define _USE_MATH_DEFINES 

typedef std::list<GameObject*> game_obj_list;
typedef std::map<const char*, game_obj_list, ltstr> game_obj_map;

vector3d rotate(vector3d v, vector3d p);

class Room {
	game_obj_map objectMap;
	GameObject floor, room;
	const Graph igraph, pgraph;
	bool addObject(const char* subj, int predicate, const char* obj);
	
	// private placement methods
	bool overlaps(GameObject* subj, GameObject* obj);
	bool canPlace(GameObject* subj);
	bool centerOf(GameObject* subj, GameObject* obj);		// match x,y
	bool edgeOf(GameObject* subj, GameObject* obj, int angle);			// match outer x,y w/ inner x,y
	bool around(GameObject* subj, GameObject* obj, int angle);			// match outer x,y w/ outer x,y
	bool cornerOf(GameObject* subj, GameObject* obj);		// match corner w/ corner
	bool below(GameObject* subj, GameObject* obj);			// match upper z w/ lower z
	bool onTopOf(GameObject* subj, GameObject* obj);		// match lower z w/ upper z
	bool inside(GameObject* subj, GameObject* obj);			// bounding box w/in bounding box
	bool beside(GameObject* subj, GameObject* obj);			// match lower z's

public:
	static int getPredicate(char* pred);
	
	Room(const Graph &ig, const Graph &pg);
	~Room();
	bool tryToPlace(const char* gameObj, const char* subj);
	const char* hasObject(const char* obj) const;
	void toRecord(CEsmRecord* pRecord) const;
};


#endif