/*===========================================================================
*
* File:	Sample.CPP
* Author:	Dave Humphrey (uesp@m0use.net)
* Created On:	May 10, 2003
*
* Sample project demonstrating use of the Morrowind CEsmFile and related
* classes. Read the sample functions below for a general idea of how to use
* the file and record classes. For details on each specific record, read
* the record/sub-record source/include files. 
*
*=========================================================================*/

/* Include Files */
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>
#include <xercesc/sax2/DefaultHandler.hpp>
#include <xercesc/util/XMLString.hpp>
#include "utils.h"
#include "container_test.h"
#include "predicate_handler.h"
#include "room.h"

using namespace std;
using namespace xercesc;

DEFINE_FILE("Sample.cpp");

/*===========================================================================
*
* Function - void LoadSaveEsp (void);
*
* Sample which simply loads an existing ESP/ESM file and saves it to 
* another file.
*
*=========================================================================*/
void LoadSaveEsp (void) {
	CEsmFile TestFile;
	CEsmFile NewFile;
	bool     Result;

	/* Read test file, abort on any error */
	Result = TestFile.Read("test.esp");
	if (!Result) return;

	/* Write file, abort on any error */
	Result = TestFile.Write("backup.esp");
	if (!Result) return;

	/* Create a new file from scratch. All existing records in the NewFile
	* are removed and the file is initialized with the required information
	* (such as the TES3 header record, etc...). The MAST records are not
	* added automatically by the CEsmFile class. */
	//NewFile.CreateNew();

}
/*===========================================================================
*		End of Function LoadSaveEsp()
*=========================================================================*/


/*===========================================================================
*
* Function - void EditRecords (void);
*
* Sample function demonstrating how to add, edit, and remove records from
* the ESP/ESM file.
*
*=========================================================================*/
void EditRecords (void) {
	CEsmFile       TestFile;
	CEsmRecord*    pRecord;
	CEsmContainer* pContainer;
	bool           Result;

	/* Read test file, abort on any error */
	Result = TestFile.Read("test.esp");
	if (!Result) return;

	/* Edit record 
	if (TestFile.GetNumRecords() > 0) {
	pRecord = TestFile.GetRecord(0);
	pRecord->SetBlocked(true);
	pRecord->SetPersist(true);
	}
	*/
	/* Add and initialize a new Container record */
	cout << "adding a new container record." << endl;
	pRecord = TestFile.AllocateRecord(MWESM_REC_CONT);
	pRecord->SetID("my_test_cont_01");
	pContainer = (CEsmContainer *) pRecord;
	pContainer->SetName("Test Container");
	pContainer->SetWeight(500);

	/* Create a copy of an existing record. The new record is not added to the
	* TestFile. It must be done manually if desired. */
	//pRecord = TestFile.CreateCopy(pContainer);
	//TestFile.AddRecord(pRecord);

	Result = TestFile.Write("backup.esp");
	if (!Result) {
		cout << "error writing file" << endl;
	}
	/* Delete the new container records */
	//TestFile.DeleteRecord(pContainer);
	//TestFile.DeleteRecord(pRecord);
}
/*===========================================================================
*		End of Function EditRecords()
*=========================================================================*/

/*===========================================================================
*
* Function - void FindRecords (void);
*
* Sample function demonstrating how to find or enumerate through the 
* records in a ESP/ESM file.
*
*=========================================================================*/
void FindRecords (void) {
	CEsmFile      TestFile;
	CEsmRecord*   pRecord;
	bool          Result; 
	int		ArrayIndex;

	/* Read test file, abort on any error */
	cout << "loading file\n";
	Result = TestFile.Read("test.esp");
	if (!Result) return;
	cout << "done loading\n";

	vector3d size = {128.0f, 128.0f, 128.0f};
	vector3d pos = {400.0f, 400.0f, 400.0f};
	vector3d rot = {0.0f, 0.0f, 0.0f};

	GameObject gameObject1(_T("in_hlaalu_hall_center"), pos, rot, size);
	CEsmSubCellRef* cref1 = gameObject1.toCREF();

	GameObject gameObject2(_T("in_hlaalu_hall_center"), pos, rot, size);
	gameObject2.putRight(&gameObject1);
	CEsmSubCellRef* cref2 = gameObject2.toCREF();

	/* Start finding all cell items. Record name definitions can be found
	* in the EsmDefs.h include file. */
	pRecord = TestFile.FindFirst(MWESM_REC_CELL, ArrayIndex);
	pRecord->AddSubRecord(cref1);
	pRecord->AddSubRecord(cref2);
	
	pos6data_t pPosData;
	pPosData.PosX = 200.0f;
	pPosData.PosY = 100.0f;
	pPosData.PosZ = pPosData.RotX = pPosData.RotY = pPosData.RotZ = 0;
	addCREF(pRecord, _T("in_hlaalu_hall_center"), &pPosData);

	while (pRecord != NULL) {
		printRecord(pRecord);
		pRecord = TestFile.FindNext(MWESM_REC_WEAP, ArrayIndex);
	}

	Result = TestFile.Write("backup.esp");
	if (!Result) {
		cout << "error writing file" << endl;
	}
}
/*===========================================================================
*		End of Function FindRecords()
*=========================================================================*/

// game_obj - the unique game object name required for object placement
// subj - the current concept
// room - the room trying to place in
// pmap - placement map that maps subj to list of pred_obj pairs
// imap - is map that maps subj to list of objects that satisfy the is a relationship
/*
bool tryToPlace(const char* gameObj, const char* subj, Room* room, const Graph& pgraph, const Graph& igraph) {
	if (!igraph.hasNode(subj)) {
		printf("error: couldn't find is_a mapping for %s\n", subj);
		return false;
	}

	edge_map isa = igraph.getAdjacent(subj);
	edge_map::const_iterator isa_iter;

	for (isa_iter = isa.begin(); isa_iter != isa.end(); isa_iter++) {
		const char* nextSubj = (*isa_iter).first;
		
		if (!pgraph.hasNode(nextSubj)) {
			// not really an error/warning
			printf("warning: couldn't find pred obj list for %s\n", nextSubj);
			continue;		
		}

		edge_map pred = pgraph.getAdjacent(nextSubj);
		edge_map::const_iterator pred_iter;

		for (pred_iter = pred.begin(); pred_iter != pred.end(); pred_iter++) {
			std::pair<const char*, int> pop = (*pred_iter);
			if (room->addObject(gameObj, pop.second, pop.first, igraph)) {
				return true;
			}
		}

		// recurse down the [isA] heirarchy
		if (tryToPlace(gameObj, nextSubj, room, pgraph, igraph)) {
			return true;
		}
	}
	return false;
}
*/
/*===========================================================================
*
* Begin Main Program
*
*=========================================================================*/
int main (int argc, char* argv[]) {
	//FindRecords();
	//run_container_test();
	//return (0);

	try {
		XMLPlatformUtils::Initialize();
	}
	catch (const XMLException& toCatch) {
		char* message = XMLString::transcode(toCatch.getMessage());
		cout << "Error during initialization! :\n";
		cout << "Exception message is: \n" << message << "\n";
		XMLString::release(&message);
		return 1;
	}

	char* xmlFile = "predicates.xml";
	SAX2XMLReader* parser = XMLReaderFactory::createXMLReader();
	parser->setFeature(XMLUni::fgSAX2CoreValidation, true);   // optional
	parser->setFeature(XMLUni::fgSAX2CoreNameSpaces, true);   // optional

	MySAX2Handler* defaultHandler = new MySAX2Handler();
	parser->setContentHandler(defaultHandler);
	parser->setErrorHandler(defaultHandler);

	try {
		parser->parse(xmlFile);
	}
	catch (const XMLException& toCatch) {
		char* message = XMLString::transcode(toCatch.getMessage());
		cout << "Exception message is: \n"
			<< message << "\n";
		XMLString::release(&message);
		return -1;
	}
	catch (const SAXParseException& toCatch) {
		char* message = XMLString::transcode(toCatch.getMessage());
		cout << "Exception message is: \n"
			<< message << "\n";
		XMLString::release(&message);
		return -1;
	}
	catch (...) {
		cout << "Unexpected Exception \n" ;
		return -1;
	}

	//"furn_com_r_table_01"
	//"misc_com_redware_vase"
	Room room(defaultHandler->igraph, defaultHandler->pgraph);
	defaultHandler->pgraph.print();
	printf("\n\n");

	if (defaultHandler->igraph.hasBackwardPath("table", "furn_com_r_table_01")) {
//		printf("found backward path!\n");
	}

	if (defaultHandler->igraph.hasForwardPath("furn_com_r_table_01", "furniture")) {
//		printf("found forward path!\n");
	}

//	printf("\n\n");

	room.tryToPlace("furn_com_r_table_01", "furn_com_r_table_01");

	if (room.hasObject("table")) {
	//	printf("successfully placed table\n");
	}

	room.tryToPlace("misc_com_redware_vase", "misc_com_redware_vase");
	room.tryToPlace("furn_com_r_chair_01", "furn_com_r_chair_01");
	room.tryToPlace("furn_com_r_chair_01", "furn_com_r_chair_01");
	room.tryToPlace("misc_imp_silverware_plate_01", "misc_imp_silverware_plate_01");
	room.tryToPlace("misc_imp_silverware_plate_01", "misc_imp_silverware_plate_01");
	room.tryToPlace("ingred_ash_yam_01", "ingred_ash_yam_01");
	room.tryToPlace("ingred_ash_yam_01", "ingred_ash_yam_01");
	room.tryToPlace("furn_com_r_bookshelf_01", "furn_com_r_bookshelf_01");
	
	CEsmFile      TestFile;
	CEsmRecord*   pRecord;
	bool          Result; 
	int		ArrayIndex;

	/* Read test file, abort on any error */
	cout << "loading file\n";
	Result = TestFile.Read("test.esp");
	if (!Result) return -1;
	cout << "done loading\n";

	
	for (ArrayIndex = 0; ArrayIndex < TestFile.GetNumRecords(); ArrayIndex++) {
		pRecord = TestFile.GetRecord(ArrayIndex);
		if (pRecord->IsType(MWESM_REC_CELL) && pRecord->GetID() == "Test Room") {
			break;
		};
	}
	if (!pRecord) {
		cout << "no record found" << endl;
		return -1;
	}
	//pRecord = TestFile.FindFirst(MWESM_REC_CELL, ArrayIndex);
	room.toRecord(pRecord);
	printRecord(pRecord);

	Result = TestFile.Write("backup.esp");
	if (!Result) {
		cout << "error writing file" << endl;
	}

	delete parser;
	delete defaultHandler;
	return 0;
}
/*===========================================================================
*		End of Main Program
*=========================================================================*/

