/*===========================================================================
 *
 * File:	Esmoptions.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	April 6, 2003
 *
 * Implements the CEsmOptions class.
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "esmoptions.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("EsmOptions.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmOptions Constructor
 *
 *=========================================================================*/
CEsmOptions::CEsmOptions () {
  //DEFINE_FUNCTION("CEsmOptions::CEsmOptions()");

	/* Initialize the script options */
  m_BlueScriptOptions.CreateDefaultBlueFormat();
  m_WhiteScriptOptions.CreateDefaultWhiteFormat();
  m_UserScriptOptions.CreateDefaultWhiteFormat();
  m_ScriptFormatType = ESMSCR_FORMAT_USER;
  m_AuthorName[0] = NULL_CHAR;

  m_NoScriptFormat  = false;
  m_ScriptWarnLevel = ESMSCR_COMPILEMSG_DEFAULT;
 }
/*===========================================================================
 *		End of Class CEsmOptions Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmOptions Method - void Destroy (void);
 *
 *=========================================================================*/
void CEsmOptions::Destroy (void) {
  //DEFINE_FUNCTION("CEsmOptions::Destroy()");

 }
/*===========================================================================
 *		End of Class Method CEsmOptions::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmOptions Method - CEsmScriptOptions* GetScriptOptions (void);
 *
 * Returns the script format options from the current type field.
 *
 *=========================================================================*/
CEsmScriptOptions* CEsmOptions::GetScriptOptions (void) {

  switch (m_ScriptFormatType) {
    case ESMSCR_FORMAT_WHITE:	return (&m_WhiteScriptOptions);
    case ESMSCR_FORMAT_BLUE:	return (&m_BlueScriptOptions);
    case ESMSCR_FORMAT_USER:
    default:			return (&m_UserScriptOptions);
   }

 }
/*===========================================================================
 *		End of Class Method CEsmOptions::GetScriptOptions()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmOptions Method - bool ReadFromRegistry (void);
 *
 * Read the options from the registry. Returns false on any error.
 *
 *=========================================================================*/
bool CEsmOptions::ReadFromRegistry (void) {
  CWinApp* pApp = AfxGetApp();
  CString  Buffer;
  bool	   Result;

	/* General options */
  Buffer = pApp->GetProfileString(ESMSCR_REGSEC_GENERAL, ESMSCR_REGENTRY_AUTHORNAME, NULL);
  SetAuthorName(Buffer);

	/* Script options */
  m_ScriptWarnLevel  = pApp->GetProfileInt(ESMSCR_REGSEC_SCRIPT, ESMSCR_REGENTRY_WARNLEVEL,    m_ScriptWarnLevel);
  m_NoScriptFormat   = (pApp->GetProfileInt(ESMSCR_REGSEC_SCRIPT, ESMSCR_REGENTRY_NOSCRFORMAT, m_NoScriptFormat) != 0);
  m_ScriptFormatType = pApp->GetProfileInt(ESMSCR_REGSEC_SCRIPT,  ESMSCR_REGENTRY_SCRFORMAT,   m_ScriptFormatType);

	/* Script user format */
  Result = m_UserScriptOptions.ReadFromRegistry();
  return (Result);
 }
/*===========================================================================
 *		End of Class Method CEsmOptions::ReadFromRegistry()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmOptions Method - bool WriteToRegistry (void);
 *
 * Writes the options to the registry. Returns false on any error.
 *
 *=========================================================================*/
bool CEsmOptions::WriteToRegistry (void) {
  CWinApp* pApp = AfxGetApp();
  bool	   Result;
  int	   iResult;

  	/* General options */
  iResult = pApp->WriteProfileString(ESMSCR_REGSEC_GENERAL, ESMSCR_REGENTRY_AUTHORNAME, m_AuthorName);
  
  	/* Script options */
  iResult &= pApp->WriteProfileInt(ESMSCR_REGSEC_SCRIPT, ESMSCR_REGENTRY_WARNLEVEL,   m_ScriptWarnLevel);
  iResult &= pApp->WriteProfileInt(ESMSCR_REGSEC_SCRIPT, ESMSCR_REGENTRY_NOSCRFORMAT, m_NoScriptFormat);
  iResult &= pApp->WriteProfileInt(ESMSCR_REGSEC_SCRIPT, ESMSCR_REGENTRY_SCRFORMAT,   m_ScriptFormatType);

	/* Script user format */
  Result = m_UserScriptOptions.WriteToRegistry();
  return (Result & (iResult != 0));
 }
/*===========================================================================
 *		End of Class Method CEsmOptions::WriteToRegistry()
 *=========================================================================*/


