/*===========================================================================
 *
 * File:	Esmoptions.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	April 6, 2003
 *
 * Defines the CEsmOptions class, used for loading/save INI files and storing
 * various user options.
 *
 *=========================================================================*/
#ifndef __ESMOPTIONS_H
#define __ESMOPTIONS_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "EsmDefs.h"
  #include "EsmOptionsDef.h"
  #include "EsmScriptOptions.h"
  #include "EsmRegDefs.h"
  #include "dl_str.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CEsmOptions Definition
 *
 * Stores various user options and handles the input/output of config files.
 *
 *=========================================================================*/
class CEsmOptions {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CEsmScriptOptions	m_BlueScriptOptions;		/* Standard/user script settings */
  CEsmScriptOptions	m_WhiteScriptOptions;
  CEsmScriptOptions	m_UserScriptOptions;
  int			m_ScriptFormatType;

  bool			m_NoScriptFormat;		/* Display script format or not? */
  int			m_ScriptWarnLevel;		/* The warning level of the compiler */

				/* Default author name for new plugins */
  TCHAR			m_AuthorName[ESMSCR_OPTION_NAMESIZE+1];


  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CEsmOptions();
  virtual ~CEsmOptions() { Destroy(); }
  virtual void Destroy (void);

	/* Get class members */
  CEsmScriptOptions*	GetScriptOptions   (void);
  CEsmScriptOptions*	GetUserFormat	   (void) { return (&m_UserScriptOptions); }
  const TCHAR*		GetAuthorName	   (void) { return (m_AuthorName); }
  bool			GetNoScriptFormat  (void) { return (m_NoScriptFormat); }
  bool			GetNoToolTips	   (void) { return (m_UserScriptOptions.GetNoToolTips()); }
  int			GetScriptWarnLevel (void) { return (m_ScriptWarnLevel); }
  int			GetFormatType	   (void) { return (m_ScriptFormatType); }

	/* Set class members */
  void SetBlueScript	  (void)	    { m_ScriptFormatType = ESMSCR_FORMAT_BLUE; }
  void SetUserScript	  (void)	    { m_ScriptFormatType = ESMSCR_FORMAT_USER; }
  void SetWhiteScript     (void)	    { m_ScriptFormatType = ESMSCR_FORMAT_WHITE; }
  void SetScriptWarnLevel (const int Value) { m_ScriptWarnLevel = Value; }
  void SetNoScriptFormat  (const bool Flag) { m_NoScriptFormat = Flag; }
  void SetNoToolTips      (const bool Flag) { m_UserScriptOptions.SetNoToolTips(Flag); }
  void SetAuthorName	  (const TCHAR* pString) { strnncpy(m_AuthorName, pString, ESMSCR_OPTION_NAMESIZE); }

	/* Read options from the registry */
  bool ReadFromRegistry (void);

	/* Write options to the registry */
  bool WriteToRegistry (void);

 };
/*===========================================================================
 *		End of Class CEsmOptions Definition
 *=========================================================================*/



#endif
/*===========================================================================
 *		End of File Esmoptions.H
 *=========================================================================*/
